# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from typing import Any

from .authorizationexception import AuthorizationException


class PolicyFailure(AuthorizationException):
    __module__: str = 'cbra.ext.oauth2.types'
    name: str
    allow: list[Any]

    def __init__(self, *, name: str, allow: list[Any] | None = None):
        self.name = name
        self.allow = allow or []
